<div>
  <sly-select-team-files
    v-if="data.{{{widget.widget_id}}}.teamId"
    :team-id.sync="data.{{{widget.widget_id}}}.teamId"
    :options="data.{{{widget.widget_id}}}.options"
    @items-selected="state.{{{widget.widget_id}}}.selected = $event"
  />
</div>
